package explorviz.live_trace_processing.filter.reduction.evaluation.ad_hoc;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import explorviz.live_trace_processing.filter.IPipeReceiver;
import explorviz.live_trace_processing.filter.reduction.AbstractReductionFilter;
import explorviz.live_trace_processing.record.event.AbstractAfterEventRecord;
import explorviz.live_trace_processing.record.event.AbstractBeforeEventRecord;
import explorviz.live_trace_processing.record.event.AbstractEventRecord;
import explorviz.live_trace_processing.record.trace.Trace;

public class FragmentationFilter extends AbstractReductionFilter {

	private final int objectId;

	/*
	 * The FragmentationFilter keeps all Records with the given objectId.
	 */
	public FragmentationFilter(final IPipeReceiver receiver, final int objectId) {
		super(receiver, "FragmentationFilter");
		this.objectId = objectId;
	}

	@Override
	protected Trace reduceTrace(final Trace trace) {
		final List<AbstractEventRecord> traceEvents = new ArrayList<AbstractEventRecord>();
		final Stack<Integer> stack = new Stack<Integer>();
		int count = 0;
		for (final AbstractEventRecord event : trace.getTraceEvents()) {
			if (event instanceof AbstractBeforeEventRecord) {
				count++;
				final int objectId = ((AbstractBeforeEventRecord) event).getObjectId();
				if (this.objectId == objectId) {
					traceEvents.add(event);
					stack.push(new Integer(count));
				}
			} else if (event instanceof AbstractAfterEventRecord) {
				if (!stack.isEmpty() && (stack.peek() == count)) {
					traceEvents.add(event);
					stack.pop();
				}
				count--;
			}
		}

		return new Trace(traceEvents, true);
	}

}
