package explorviz.live_trace_processing.filter.reduction.evaluation.ad_hoc;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import explorviz.live_trace_processing.filter.IPipeReceiver;
import explorviz.live_trace_processing.filter.reduction.AbstractReductionFilter;
import explorviz.live_trace_processing.record.event.AbstractAfterEventRecord;
import explorviz.live_trace_processing.record.event.AbstractAfterFailedEventRecord;
import explorviz.live_trace_processing.record.event.AbstractBeforeEventRecord;
import explorviz.live_trace_processing.record.event.AbstractEventRecord;
import explorviz.live_trace_processing.record.trace.Trace;

public class SamplingFilter extends AbstractReductionFilter {

	private final int samplingDistance;

	public SamplingFilter(final IPipeReceiver receiver, final int samplingDistance) {
		super(receiver, "Sampling");
		this.samplingDistance = samplingDistance;
	}

	@Override
	protected Trace reduceTrace(final Trace trace) {
		final List<AbstractEventRecord> newTraceEvents = new ArrayList<AbstractEventRecord>();
		int stackDepth = 0;
		int counter = samplingDistance;
		final Stack<Integer> stack = new Stack<Integer>();
		for (final AbstractEventRecord record : trace.getTraceEvents()) {
			if (record instanceof AbstractBeforeEventRecord) {
				stackDepth++;
				if (counter == samplingDistance) {
					newTraceEvents.add(record);
					stack.push(new Integer(stackDepth));
					counter = 0;
				}
				counter++;
			} else if ((record instanceof AbstractAfterEventRecord)
					|| (record instanceof AbstractAfterFailedEventRecord)) {
				if (!stack.isEmpty() && (stackDepth == stack.peek())) {
					stack.pop();
					newTraceEvents.add(record);
				}
				stackDepth--;
			} else {
				newTraceEvents.add(record);
			}
		}
		return new Trace(newTraceEvents, true);
	}

}
