package explorviz.live_trace_processing.filter.reduction.evaluation.summarization;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import explorviz.live_trace_processing.filter.IPipeReceiver;
import explorviz.live_trace_processing.filter.reduction.AbstractReductionFilter;
import explorviz.live_trace_processing.record.event.AbstractAfterEventRecord;
import explorviz.live_trace_processing.record.event.AbstractBeforeEventRecord;
import explorviz.live_trace_processing.record.event.AbstractEventRecord;
import explorviz.live_trace_processing.record.trace.Trace;

public class MonotoneSubsequenceFilter extends AbstractReductionFilter {

	public MonotoneSubsequenceFilter(final IPipeReceiver receiver) {
		super(receiver, "MonotoneSubsequence");
	}

	@Override
	protected Trace reduceTrace(final Trace trace) {
		final List<AbstractEventRecord> newTraceEvents = new ArrayList<AbstractEventRecord>();
		int count = 0;
		boolean keepNextBeforeEvent = true;
		boolean lastEventWasAfterEvent = false;
		final Stack<Integer> searchCorrespondingAfterEvent = new Stack<Integer>();
		for (final AbstractEventRecord event : trace.getTraceEvents()) {
			if (event instanceof AbstractBeforeEventRecord) {
				count++;
				lastEventWasAfterEvent = false;
				if (keepNextBeforeEvent) {
					newTraceEvents.add(event);
					searchCorrespondingAfterEvent.push(new Integer(count));
					keepNextBeforeEvent = false;
				}
			} else if (event instanceof AbstractAfterEventRecord) {
				if (lastEventWasAfterEvent) {
					keepNextBeforeEvent = true;
				} else {
					lastEventWasAfterEvent = true;
				}
				if (!searchCorrespondingAfterEvent.isEmpty()
						&& (count == searchCorrespondingAfterEvent.peek())) {
					searchCorrespondingAfterEvent.pop();
					newTraceEvents.add(event);
				}
				count--;
			} else {
				newTraceEvents.add(event);
			}
		}
		return new Trace(newTraceEvents, true);
	}

}
