package explorviz.live_trace_processing.filter.reduction.summarization;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import explorviz.live_trace_processing.filter.IPipeReceiver;
import explorviz.live_trace_processing.filter.reduction.AbstractReductionFilter;
import explorviz.live_trace_processing.record.event.AbstractAfterEventRecord;
import explorviz.live_trace_processing.record.event.AbstractBeforeEventRecord;
import explorviz.live_trace_processing.record.event.AbstractEventRecord;
import explorviz.live_trace_processing.record.trace.Trace;

public class PatternSummarizationFilter extends AbstractReductionFilter {

	public static final int DEFAULT_DISTANCE = 300;
	public static final int DEFAULT_NESTINGLEVEL = 1;
	private final int distance;
	private final int nestingLevel;

	// private int maxDistance = 0;
	// private boolean nextRound = false;
	// private int maxNestingLevel = 0;

	public PatternSummarizationFilter(final IPipeReceiver receiver, final int distance,
			final int nestingLevel) {
		super(receiver, "PatternSummarization");
		if (distance < 1) {
			this.distance = DEFAULT_DISTANCE;
		} else {
			this.distance = distance;
		}
		if (nestingLevel < 1) {
			this.nestingLevel = DEFAULT_NESTINGLEVEL;
		} else {
			this.nestingLevel = nestingLevel;
		}
	}

	@Override
	protected Trace reduceTrace(Trace trace) {
		int t = 0;
		while (t < nestingLevel) {
			trace = summarizePattern(trace);
			// if (!nextRound) {
			// System.out.println("max distance: " + maxDistance);
			// System.out.println("max nesting level: " + maxNestingLevel);
			// return trace;
			// }
			// nextRound = false;
			// maxNestingLevel++;
			t++;
		}
		return trace;
	}

	private Trace summarizePattern(final Trace trace) {
		final List<AbstractEventRecord> newTraceEvents = new ArrayList<AbstractEventRecord>();
		final List<AbstractEventRecord> traceEvents = trace.getTraceEvents();
		int index = 0;
		final int traceLength = traceEvents.size();
		int count = 0;
		final Stack<Integer> stack = new Stack<Integer>();

		while (index < traceLength) {
			final AbstractEventRecord event = traceEvents.get(index);
			if (event instanceof AbstractBeforeEventRecord) {
				count++;
				final AbstractBeforeEventRecord beforeEvent = (AbstractBeforeEventRecord) event;
				final int possibleRepititionStart = findPatternStart(beforeEvent, traceEvents,
						index, traceLength);
				if (possibleRepititionStart != -1) {
					final int numberOfPatternRepititions = findPatternRepititions(traceEvents,
							index, possibleRepititionStart, traceLength);
					if (numberOfPatternRepititions != 0) {
						// nextRound = true;
						removeRepititions(index, numberOfPatternRepititions,
								possibleRepititionStart, traceEvents, newTraceEvents);
						count--;
						index += (numberOfPatternRepititions + 1)
								* (possibleRepititionStart - index);
					} else {
						newTraceEvents.add(event);
						stack.push(new Integer(count));
						index++;
					}
				} else {
					newTraceEvents.add(event);
					stack.push(new Integer(count));
					index++;
				}
			} else if (event instanceof AbstractAfterEventRecord) {
				if (!stack.isEmpty() && (stack.peek() == count)) {
					stack.pop();
					newTraceEvents.add(event);
				}
				index++;
				count--;
			} else {
				newTraceEvents.add(event);
				index++;
			}
		}

		return new Trace(newTraceEvents, true);
	}

	// summarize a found pattern
	private void removeRepititions(final int start, final int numberOfPatternRepititions,
			final int end, final List<AbstractEventRecord> traceEvents,
			final List<AbstractEventRecord> newTraceEvents) {
		final int d = end - start;
		// if (d > maxDistance) {
		// maxDistance = d;
		// }
		for (int i = start; i < end; i++) {
			final AbstractEventRecord event = traceEvents.get(i);
			if (event instanceof AbstractBeforeEventRecord) {
				final AbstractBeforeEventRecord beforeEvent = (AbstractBeforeEventRecord) event;
				for (int r = 1; r <= numberOfPatternRepititions; r++) {
					final AbstractBeforeEventRecord nextEqualEvent = (AbstractBeforeEventRecord) traceEvents
							.get(i + (r * d));
					beforeEvent.getRuntimeStatisticInformation().merge(
							nextEqualEvent.getRuntimeStatisticInformation());

				}
			}
			newTraceEvents.add(event);
		}

	}

	// check if a pattern repetition is present, i.e., t[i+1]=t[j+1] ...
	private int findPatternRepititions(final List<AbstractEventRecord> traceEvents, int i, int j,
			final int traceLength) {
		int numberOfRepititions = 0;
		final int d = j - i;
		int counter = 0;
		while (j < traceLength) {
			final AbstractEventRecord event1 = traceEvents.get(i);
			final AbstractEventRecord event2 = traceEvents.get(j);
			if (event1.compareTo(event2) != 0) {
				return numberOfRepititions;
			}
			counter++;
			if (counter == d) {
				numberOfRepititions++;
				counter = 0;
			}
			i++;
			j++;
		}
		return numberOfRepititions;
	}

	// search j with trace[i]==trace[j] and j-i<=d and equal caller(count==0)
	// -1: no hit
	private int findPatternStart(final AbstractBeforeEventRecord beforeEvent,
			final List<AbstractEventRecord> traceEvents, int i, final int traceLength) {
		int end = i + distance;
		if (end >= traceLength) {
			end = traceLength - 1;
		}
		int count = 0;
		while (i < end) {
			i++;
			final AbstractEventRecord event = traceEvents.get(i);
			if (event instanceof AbstractBeforeEventRecord) {
				count++;
				if ((count == 0) && (beforeEvent.compareTo(event) == 0)) {
					return i;
				}
			} else if (event instanceof AbstractAfterEventRecord) {
				count--;
			}
		}
		return -1;
	}

}
